package com.zanxiang.manage.service.Impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zanxiang.common.enums.GameCategoryEnum;
import com.zanxiang.common.utils.bean.BeanUtils;
import com.zanxiang.manage.domain.dto.GameDTO;
import com.zanxiang.manage.domain.params.GameListParam;
import com.zanxiang.manage.domain.vo.*;
import com.zanxiang.manage.service.GameCategoryService;
import com.zanxiang.manage.service.GamePayWayService;
import com.zanxiang.manage.service.GameService;
import com.zanxiang.manage.service.GameStrategyService;
import com.zanxiang.mybatis.entity.Game;
import com.zanxiang.mybatis.mapper.GameMapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @author : lingfeng
 * @time : 2022-06-24
 * @description : 游戏逻辑
 */
@Slf4j
@Service
public class GameServiceImpl extends ServiceImpl<GameMapper, Game> implements GameService {

    @Autowired
    private GamePayWayService gamePayWayService;

    @Autowired
    private GameStrategyService gameStrategyService;

    @Autowired
    private GameCategoryService gameCategoryService;

    /**
     * 查询游戏列表
     *
     * @param param : 游戏列表查询参数
     * @return : 返回单页游戏游戏列表数据
     */
    @Override
    public IPage<GameListVO> gameList(GameListParam param) {
        //查询游戏类别
        List<GameCategoryVO> gameCategoryVOList = new ArrayList<>();
        if (Objects.equals(param.getIsParentClassify(), Boolean.TRUE)) {
            gameCategoryVOList = gameCategoryService.listById(param.getGameClassifyId(), GameCategoryEnum.GAME_LABEL.getCategoryType());
        }
        //执行查询
        return page(param.toPage(), new QueryWrapper<Game>().lambda()
                .eq(param.getCpId() != null, Game::getCpId, param.getCpId())
                .eq(param.getGameId() != null && !Objects.equals(param.getIsParentGame(), Boolean.TRUE), Game::getId, param.getGameId())
                .and(param.getGameId() != null && Objects.equals(param.getIsParentGame(), Boolean.TRUE),
                        qw -> qw.eq(Game::getId, param.getGameId()).or().eq(Game::getParentId, param.getGameId()))
                .eq(param.getGameCategoryId() != null, Game::getCategory, param.getGameCategoryId())
                .eq(param.getH5GameId() != null, Game::getH5GameId, param.getH5GameId())
                .eq(param.getGuideGameId() != null, Game::getGuideGameId, param.getGuideGameId())
                .eq(param.getGameClassifyId() != null && !Objects.equals(param.getIsParentClassify(), Boolean.TRUE), Game::getClassify, param.getGameClassifyId())
                .in(param.getGameClassifyId() != null && !gameCategoryVOList.isEmpty(), Game::getClassify, param.getGameClassifyId())
                .eq(param.getStatus() != null, Game::getStatus, param.getStatus())
                .orderByDesc(Game::getCreateTime)
        ).convert(this::toVo);
    }

    /**
     * 游戏对象信息转换
     *
     * @param game : 游戏信息
     * @return : 返回展示对象
     */
    private GameListVO toVo(Game game) {
        if (Objects.isNull(game)) {
            return null;
        }
        GameListVO gameListVO = BeanUtils.copy(game, GameListVO.class);
        gameListVO.setIsParentGame(Objects.equals(game.getParentId(), 0L));
        //查询支付方式列表
        List<GamePayWayVO> gamePayWayVOList = gamePayWayService.getByGameId(game.getId());
        gameListVO.setGamePayWayVOList(gamePayWayVOList);
        //查询规则配置列表
        List<GameStrategyVO> gameStrategyVOList = gameStrategyService.getByGameId(game.getId());
        gameListVO.setGameStrategyVOList(gameStrategyVOList);
        return gameListVO;
    }

    /**
     * 游戏选择列表查询
     *
     * @return : 返回所有游戏信息
     */
    @Override
    public List<GameChoiceVO> choiceList() {
        List<Game> gameList = super.list(new LambdaQueryWrapper<Game>().select(Game::getId, Game::getName));
        return BeanUtils.copyList(gameList, GameChoiceVO.class);
    }

    /**
     * 根据id查询游戏信息
     *
     * @param id : 游戏id
     * @return : 返回游戏信息
     */
    @Override
    public GameDTO getById(Long id) {
        if (id == null) {
            return null;
        }
        return BeanUtils.copy(super.getById(id), GameDTO.class);
    }

    /**
     * 游戏相关条件查询游戏信息
     *
     * @param cpId           : cp唯一键
     * @param gameId         : 游戏id
     * @param gameCategoryId : 分类id
     * @return : 返回游戏信息列表
     */
    @Override
    public Map<Long, GameDTO> gameCondition(Long cpId, Long gameId, Long gameCategoryId) {
        //游戏条件判断
        if (cpId == null && gameId == null && gameCategoryId == null) {
            return null;
        }
        List<GameDTO> gameList = this.getGameList(cpId, gameId, gameCategoryId);
        if (CollectionUtils.isEmpty(gameList)) {
            return Collections.emptyMap();
        }
        return gameList.stream().collect(Collectors.toMap(GameDTO::getId, Function.identity()));
    }

    /**
     * 游戏查询条件
     *
     * @param cpId           : cp唯一键
     * @param gameId         : 游戏id
     * @param gameCategoryId : 分类id
     * @return : 返回游戏信息列表, null 标识未匹配到游戏, 空表示没有条件
     */
    @Override
    public List<GameDTO> getGameList(Long cpId, Long gameId, Long gameCategoryId) {
        if (cpId == null && gameId == null && gameCategoryId == null) {
            return Collections.emptyList();
        }
        List<Game> gameList = super.list(new LambdaQueryWrapper<Game>()
                .eq(cpId != null, Game::getCpId, cpId)
                .eq(gameId != null, Game::getId, gameId)
                .eq(gameCategoryId != null, Game::getCategory, gameCategoryId)
                .select(Game::getId, Game::getCpId, Game::getName)
        );
        return BeanUtils.copyList(gameList, GameDTO.class);
    }

}
